# This file is automatically generated, DO NOT EDIT
# fmt: off

from os.path import abspath, join, dirname
_root = abspath(dirname(__file__))

# runtime dependencies
import wpilib._init_wpilib
import wpilib.geometry._init_geometry
import wpilib.interfaces._init_interfaces
import wpilib._impl._init_wpilibc
import hal._init_wpiHal
import wpiutil._init_wpiutil
import _pyntcore._init_ntcore
